/*
 * Copyright (c) 1998 by Tsuyoshi Iguchi (zinnia@jan.ne.jp)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: playwav.orig.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */
/*
 * playwav -- play WAV and CD-DA audio file
 */
#include <unistd.h>
#include <sys/ioccom.h>
#include <soundcard.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#define __BUFSIZ 0x1000

char buf[__BUFSIZ];
char* dspdev = "/dev/audio0";

#if VERBOSE
int verbose = 1;
#else
int verbose = 0;
#endif

int sigflag = 0;

typedef struct{
  int channel;
  int rate;
  int bits;
} WAV;

WAV orig, act;
int iswav = 0;
int dsp;

void verbose_out(char* s)
{
  if(verbose){
    fputs(s, stderr);
  }
}

void sigint(int sig)
{
/*  fprintf(stderr, "SIGNAL"); */
  verbose_out("SIGINT caught");
  verbose_out("send SOUND_PCM_RESET");
  ioctl(dsp, SOUND_PCM_RESET, 0);
  verbose_out("Ok.");
  sigflag = 1;
}

void playwav(char* name){
  int i, fd;
  int ch;

  if(!strcmp(name, "-")){
    fd = fileno(stdin);
    name = "stdin";
  } else {
    fd = open(name, O_RDONLY);
    if(fd <= 0){
      fprintf(stderr, "%s: open error", name);
      return;
    }
  }
  i = read(fd, buf, 44);
  if(i == 44){
    if(!strncmp(&buf[0], "RIFF", 4)){
      if(!strncmp(&buf[8], "WAVE", 4)){
	if(buf[20] != 1){
	  fprintf(stderr, "unexpected format type(%d)\n", buf[21]);
	  exit(1);
	}
	orig.channel = (*(unsigned short*)&buf[22]);
	orig.rate = (*(unsigned long*)&buf[24]);
	orig.bits = (*(unsigned short*)&buf[34]);
	iswav = 1;
	fprintf(stderr,
		"%s: WAV file (%dHz, %dbit, %dch)\n",
		name, orig.rate, orig.bits, orig.channel);
      }
    } 
  }
  if(!iswav){ /* CD-DA */
    lseek(fd, 0, SEEK_SET);
    orig.channel = 2;   /* stereo */
    orig.rate = 48000;  /* 44.1kHz */
    orig.bits = 16;     /* 16bit */
    fprintf(stderr,
	    "%s: CD-DA file(guess) (48000Hz, 16bit, 2ch)\n",
	    name);
  }
  act = orig;
  dsp = open(dspdev, O_RDWR);
  if(dsp <= 0){
    perror("open: dspdev"); exit(1);
  }
  if(ioctl(dsp, SOUND_PCM_WRITE_BITS, &act.bits) == -1){
    perror("SOUND_PCM_WRITE_BITS"); exit(1);
  }
  if(ioctl(dsp, SOUND_PCM_WRITE_CHANNELS, &act.channel) == -1){
    perror("SOUND_PCM_WRITE_CHANNELS"); exit(1);
  }
  if(ioctl(dsp, SOUND_PCM_WRITE_RATE, &act.rate) == -1){
    perror("SOUND_PCM_WRITE_RATE"); exit(1);
  }
#if 0
  if(act.rate != orig.rate){
    fprintf(stderr, "act = %d, orig = %d\n", act.rate, orig.rate);
    exit(1);
  }
#endif
  signal(SIGINT, sigint);
  while((i = read(fd, buf, __BUFSIZ)) > 0){
    if(sigflag) break;
    write(dsp, buf, i);
    if(sigflag) break;
  }
  ioctl(dsp, SOUND_PCM_SYNC, 0);
  close(dsp);
}

int main(int argc, char** argv)
{
  int i;
  for(i = 1; i < argc; i++){
    playwav(argv[i]);
    if(sigflag) break;
  }
}
