/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: converter.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */

#include "converter.h"

// XXX TODO: support src == dst 

void swap16(char* ptr, int len)
{
  char* p;
  int i;
  for(i = 0, p = ptr; i < len / 2; i++, p += 2){
    byteswap2(p);
  }
}

#define UNSIGNED2SIGNED(type,src,dst,len,bias) \
{ \
unsigned type* p; type* q; \
int i, t; \
p = (type*)src; q = (type*)dst; \
for(i = 0; i < len / sizeof(type); i++, p++, q++) \
{\
  t = (*p) - bias; \
  *q = (type)t; \
} \
}

void unsigned2signed8(char* src, char* dst, int len)
{
  UNSIGNED2SIGNED(char, src, dst, len, 128);
}

void unsigned2signed16(char* src, char* dst, int len)
{
  UNSIGNED2SIGNED(short, src, dst, len, 32768);
}

void bit_8to16(char* src, char* dst, int len)
{
  int i, t;
  char* p; short* q;
  p = src; q = (short*)dst;

  for(i = 0; i < len; i++, p++, q++){
    t = (*p) * 256;
    *q = t;
  }
}

#define EXPAND_STEREO(type,src,dst,len) \
{ \
type* p; type* q; \
int i; \
p = (type*)src; q = (type*)dst; \
for(i = 0; i < len / sizeof(type); i++, p++){ \
  *q = *p; q++; \
  *q = *p; q++; \
}\
}

void expand_stereo8(char* src, char* dst, int len)
{
  EXPAND_STEREO(char,src,dst,len);
}

void expand_stereo16(char* src, char* dst, int len)
{
  EXPAND_STEREO(short,src,dst,len);
}

#define JOINT_STEREO(type,src,dst,len) \
{\
type* p; type* q; \
int i, t; \
p = (type*)src; q = (type*)dst; \
for(i = 0; i < len / sizeof(type); i++, p += 2, q++){ \
t = ((*p) + (*p + 1)) / 2; \
*q = t; \
}\
}


void joint_stereo8(char* src, char* dst, int len)
{
  JOINT_STEREO(char,src,dst,len);
}

void joint_stereo16(char* src, char* dst, int len)
{
  JOINT_STEREO(short,src,dst,len);
}
