/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_libao.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */
#ifdef USE_LIBAO
#include "playwav.h"
#include <stdio.h>
#include <ao/ao.h>
#include <string.h>

static ao_device* device;

static int audio_open(int bit, int rate, int ch)
{
  int driverid;
  ao_sample_format spec;

  ao_initialize();
  driverid = ao_default_driver_id();
  if(driverid == -1) return -1;

  spec.rate = rate;
  spec.channels = ch;
  spec.bits = bit;
  spec.byte_format = AO_FMT_NATIVE;

  device = ao_open_live(driverid, &spec, NULL);
  if(device == NULL) return -1;

  return 0;
}

static int audio_play(unsigned char* buf, int len)
{
  if(ao_play(device, buf, len) == 0) return 0;
  return len;
}

static void audio_close()
{
  ao_close(device);
  ao_shutdown();
}

static void audio_sync()
{
}

driver_vector libao_vector = {
  "libao",
  audio_open,
  audio_play,
  audio_sync,
  audio_close,
};

#endif
