/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_sdl.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */
#ifdef USE_SDL
#include <SDL.h>
#include <string.h>
#include "playwav.h"
#include "ringbuffer.h"

#define CALLBACK_SAMPLESIZE 2048
#define CALLBACK_BUFFERSIZE 2048 * 4
#define RINGBUFFER_SIZE CALLBACK_BUFFERSIZE * 32

static ringbuffer_context ring;

static void mycallback(void* userdata, Uint8* stream, int len)
{
  static char buf[CALLBACK_BUFFERSIZE];
  int readlength = len;

  if(readlength > CALLBACK_BUFFERSIZE) readlength = CALLBACK_BUFFERSIZE;
  readlength = ringbuffer_get(&ring, buf, readlength);

  if(readlength > 0) {
    SDL_MixAudio(stream, buf, readlength, SDL_MIX_MAXVOLUME);
  }
}

static int audio_open(int bit, int rate, int ch)
{
  SDL_AudioSpec spec;
  if(ringbuffer_init(&ring, RINGBUFFER_SIZE) != 0){
    fprintf(stderr, "Init ringbuffer failed.\n");
    return 1;
  }
  ringbuffer_register_blocking(&ring, RINGBUFFER_BLOCKING_PUT);
  ringbuffer_register_blocking_vector(&ring, (void (*)(int))SDL_Delay);

  if(SDL_Init(SDL_INIT_AUDIO) < 0){
    fprintf(stderr, "SDL_Init: %s\n", SDL_GetError());
    return -1;
  }

  spec.freq = rate;
  spec.channels = ch;
  switch(bit){
  case 8:
    spec.format = AUDIO_S8;
    break;
  case 16:
    /* FALLTHROUGH */
  default:
    spec.format = AUDIO_S16SYS;
    break;
  }

  spec.samples = CALLBACK_SAMPLESIZE;
  spec.callback = mycallback;

  if(SDL_OpenAudio(&spec, NULL) < 0){
    fprintf(stderr, "SDL_OpenAudio: %s\n", SDL_GetError());
    return 1;
  }
  SDL_PauseAudio(0);
  return 0;
}

static int audio_play(unsigned char* buf, int len)
{
  return ringbuffer_put(&ring, buf, len);
}

static void audio_close()
{
  SDL_CloseAudio();
  ringbuffer_terminate(&ring);
}

static void audio_sync()
{
  while(ring.inlen > 0){
    SDL_Delay(50);
  }
}

driver_vector sdl_vector = {
  "sdl",
  audio_open,
  audio_play,
  audio_sync,
  audio_close,
};

#endif
