/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: drivers.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */
#include "playwav.h"
#include "drivers.h"
#include <stdio.h>

driver_vector* drivers[] = {
#ifdef USE_LIBAO
  &libao_vector,
#endif

#ifdef USE_SDL
  &sdl_vector,
#endif
  0,
};

driver_vector* query_driver(char* name)
{
  driver_vector** p;
  for(p = drivers; *p != 0; p++){
    if(strcmp(name, (*p)->name) == 0) break;
  }
  return *p;
}

void usage()
{
  fprintf(stderr, "usage: playwav [-d driver] [-l loopcount] [-f format] [-r rate] [-c ch] [filename]\n"
	  "format\n"
	  "\tu8: unsigned 8bit\n"
	  "\ts8: signed 8bit\n"
	  "\tu16|u16le: unsigned 16bit endian = little\n"
	  "\ts16|s16le: signed 16bit endian = little\n"
	  "\tu16be: unsigned 16bit endian = big\n"
	  "\ts16be: signed 16bit endian = big\n"
	  "driver\n"
	  );
  {
    driver_vector** p;
    for(p = drivers; *p != NULL; p++){
      fprintf(stderr, "\t%s", (*p)->name);
      if(strcmp((*p)->name, DEFAULT_PLAY_DRIVER) == 0){
	fprintf(stderr, " (default)");
      }
      fprintf(stderr, "\n");
    }
  }
  exit(1);
}


