/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: endian.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */

#include "endian.h"

inline void byteswap(char* p1, char* p2)
{
  register char tmp;
  tmp = *p1;
  *p1 = *p2;
  *p2 = tmp;
  return;
}

inline void byteswap2(char* ptr)
{
  byteswap(ptr, ptr + 1);
}

inline void byteswap3(char* ptr)
{
  byteswap(ptr, ptr + 2);
}
		       
inline void byteswap4(char* ptr)
{
  byteswap(ptr, ptr + 3);
  byteswap(ptr + 1, ptr + 2);
}

inline void byteswapn(char* ptr, int n)
{
  int i;
  for(i = 0; i < n / 2; i++){
    byteswap(ptr + i, ptr + n - i - 1);
  }
}

enum_endian get_systemendian()
{
  short a = 1;
  char* p = (char*)&a;
  return *p == 0 ? BIGENDIAN : LITTLEENDIAN;
}

static short endian_le2sys16little(short t){ return t; }
static int endian_le2sys32little(int t){ return t; }
static short endian_be2sys16little(short t){
  short q = t;
  byteswap2((char*)&q);
  return q;
}
static int endian_be2sys32little(int t){
  int q = t;
  byteswap4((char*)&q);
  return q;
}

static short endian_le2sys16big(short t){
  short q = t;
  byteswap2((char*)&q);
  return q;
}
static int endian_le2sys32big(int t){
  int q = t;
  byteswap4((char*)&q);
  return q;
}
static short endian_be2sys16big(short t){ return t; }
static int endian_be2sys32big(int t){ return t; }

#define REGISTER_ENDIANCONVERTER(ptr,suffix) \
  ptr->le2sys16 = endian_le2sys16##suffix; \
  ptr->le2sys32 = endian_le2sys32##suffix; \
  ptr->be2sys16 = endian_be2sys16##suffix; \
  ptr->be2sys32 = endian_be2sys32##suffix;

void get_endianconverter(endian_converter* p)
{
  if(get_systemendian() == BIGENDIAN){
    REGISTER_ENDIANCONVERTER(p, big);
  } else {
    REGISTER_ENDIANCONVERTER(p, little);
  }
}
