/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: ringbuffer.h,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */

#ifndef _RINGBUFFER_H
#define _RINGBUFFER_H

typedef struct {
  int size;
  int rpos;
  int wpos;
  int inlen;
  void (*delayfunction)(int msec);
  int blocking_bitmap;
  unsigned char* buf;
} ringbuffer_context;

#define RINGBUFFER_BLOCKING_NONE 0
#define RINGBUFFER_BLOCKING_GET 1
#define RINGBUFFER_BLOCKING_PUT 2
#define RINGBUFFER_BLOCKING_BOTH 3
#define RINGBUFFER_BLOCKING_ERROR -1

extern int ringbuffer_init(ringbuffer_context* context, int s);
extern void ringbuffer_register_blocking_vector(ringbuffer_context* context, void (*dl)(int msec));
extern int ringbuffer_register_blocking(ringbuffer_context* context, int bmode);
extern void ringbuffer_terminate(ringbuffer_context* context);
extern int ringbuffer_get(ringbuffer_context* context, unsigned char* buf, int size);
extern int ringbuffer_put(ringbuffer_context* context, unsigned char* buf, int size);

#endif
