/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: testringbuffer.c,v 1.1.1.1 2002/07/30 11:02:37 zinnia Exp $
 */

#include "ringbuffer.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "CUnit/Automated.h"

#define ASSERT_EQUALS_STR(expected,actual,len) if(memcmp(expected,actual,len) != 0){ \
char buf##__LINE__[0x100]; \
sprintf(buf##__LINE__, "expected: '%s', actual: '%s'\n", expected, actual); \
assertImplementation(0, __LINE__, buf##__LINE__, __FILE__, ""); \
    }

void test1()
{
  ringbuffer_context ring;
  unsigned char buf[11];

  ringbuffer_init(&ring, 10);
  ASSERT_EQUALS_STR("\0\0\0\0\0\0\0\0\0\0", ring.buf, 10);
  ASSERT(ring.size == 10);

  ringbuffer_register_blocking(&ring, RINGBUFFER_BLOCKING_NONE);

  ASSERT(ringbuffer_put(&ring, "ABC", 3) == 3);
  ASSERT_EQUALS_STR("ABC\0\0\0\0\0\0\0", ring.buf, 10);
  ASSERT(ring.inlen == 3);
  ASSERT(ring.rpos == 0);
  ASSERT(ring.wpos == 3);

  ASSERT(ringbuffer_get(&ring, buf, 2) == 2);
  ASSERT_EQUALS_STR(buf, "AB", 2);
  ASSERT(ring.rpos = 2);
  ASSERT(ring.wpos == 3);
  ASSERT(ring.inlen = 1);

  ASSERT(ringbuffer_put(&ring, "DEFG", 4) == 4);
  ASSERT_EQUALS_STR("ABCDEFG\0\0\0", ring.buf, 10);
  ASSERT(ring.rpos == 2);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 5);

  ASSERT(ringbuffer_get(&ring, buf, 10) == 5);
  ASSERT_EQUALS_STR("CDEFG", buf, 5);
  ASSERT(ring.rpos == 7);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 0);

  ASSERT(ringbuffer_put(&ring, "123456", 6) == 6);
  ASSERT_EQUALS_STR("456DEFG123", ring.buf, 10);
  ASSERT(ring.rpos == 7);
  ASSERT(ring.wpos == 3);
  ASSERT(ring.inlen == 6);

  ASSERT(ringbuffer_put(&ring, "98765", 5) == 4);
  ASSERT_EQUALS_STR("4569876123", ring.buf, 10);
  ASSERT(ring.rpos == 7);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 10);

  ASSERT(ringbuffer_put(&ring, "A", 1) == 0);
  ASSERT_EQUALS_STR("4569876123", ring.buf, 10);
  ASSERT(ring.rpos == 7);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 10);

  ASSERT(ringbuffer_get(&ring, buf, 4) == 4);
  ASSERT_EQUALS_STR("1234", buf, 4);
  ASSERT_EQUALS_STR("4569876123", ring.buf, 10);
  ASSERT(ring.rpos == 1);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 6);

  ASSERT(ringbuffer_get(&ring, buf, 10) == 6);
  ASSERT_EQUALS_STR("569876", buf, 6);
  ASSERT_EQUALS_STR("4569876123", ring.buf, 10);
  ASSERT(ring.rpos == 7);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 0);

  ASSERT(ringbuffer_get(&ring, buf, 10) == 0);
  ASSERT_EQUALS_STR("4569876123", ring.buf, 10);
  ASSERT(ring.rpos == 7);
  ASSERT(ring.wpos == 7);
  ASSERT(ring.inlen == 0);

}

int init(){return 0; }
int clean(){return 0; }

int main()
{
  PTestGroup testgroup;
  PTestCase testcase;
  if(initialize_registry()){
    printf("initializing registry failed.\n");
    return 0;
  }

  testgroup = add_test_group("Test Group for ringbuffer", init, clean);
  testcase = add_test_case(testgroup, "Test case for ringbuffer", test1);
  set_output_filename("Ringbuffer");
  automated_run_tests();
  cleanup_registry();
  return 0;
}
